/*---------------------------------------------------------------------------*
 *
 * Project:	fXPlayMidi
 *
 * $RCSfile: options.c,v $
 * $Date: 1997/04/26 08:20:28 $
 * $Author: klmcw $
 * $Revision: 1.3 $
 *
 *---------------------------------------------------------------------------*
 * $Log: options.c,v $
 * Revision 1.3  1997/04/26 08:20:28  klmcw
 * Removed support for Device option.
 *
 * Revision 1.2  1996/03/09 08:22:58  klmcw
 * Added firsttime special case to prompt for options when they don't exist yet.
 * Added code to save album upon exit if it has changed.
 *
 * Revision 1.1  1996/02/26 02:24:50  klmcw
 * Initial revision
 *
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
#include <fcntl.h>
#include <limits.h>
#include <memory.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "fXPlayMidi.h"
#include "album.h"
#include "options.h"
#include "path.h"

typedef struct {
  char Album[_POSIX_PATH_MAX+1];
  char Dir[_POSIX_PATH_MAX+1];
  char BaseCmd[_POSIX_PATH_MAX+1];
  char ResetCmd[_POSIX_PATH_MAX+1];
  char Filler;			/* not used anymore. was Device option. */
  int Random;
  int Repeat;
} options_t;

typedef struct {
  char Dev;
  char *Descrip;
} optionsChc_t;

#define OPTCPY(D,S) \
if(S && *S) strcpy(D, S); \
else        memset(D, '\0', sizeof(D));

/*---------------------------------------------------------------------------*/

const char *optionsFile = "options";
static int optionsFileUsePath = 1, options1stTime = 0;
static options_t Options = { "New", "", "playmidi", "resetmidi", '\0', 1, 1 };

/*---------------------------------------------------------------------------*/
const char *
optionsAlbumGet(void)
{
  return(Options.Album);
}
/*---------------------------------------------------------------------------*/
void
optionsAlbumSet(const char *Album)
{
  OPTCPY(Options.Album, Album);
}
/*---------------------------------------------------------------------------*/
const char *
optionsBaseCmdGet(void)
{
  return(Options.BaseCmd);
}
/*---------------------------------------------------------------------------*/
void
optionsBaseCmdSet(const char *BaseCmd)
{
  OPTCPY(Options.BaseCmd, BaseCmd);
}
/*---------------------------------------------------------------------------*/
const char *
optionsDirGet(void)
{
  return(Options.Dir);
}
/*---------------------------------------------------------------------------*/
void
optionsDirSet(const char *Dir)
{
  OPTCPY(Options.Dir, Dir);
}
/*---------------------------------------------------------------------------*/
int
optionsFileCreate(void)
{
  register int rc = 0;
  const char *FileNm = optionsFileNameGet();
  FILE *Fp;

  if((Fp = fopen(FileNm, "w")) == NULL) {
    printf("Could not create options file - %s!\n", FileNm);
    return(-1);
  }

  rc = (fwrite(&Options, sizeof(Options), 1, Fp) != 1);
  fclose(Fp);

  return(rc);
}
/*---------------------------------------------------------------------------*/
const char *
optionsFileNameGet(void)
{
  static char OptFile[_POSIX_PATH_MAX+1] = { '\0' };

  if(!*OptFile || !optionsFileUsePath) {
    memset(OptFile, '\0', sizeof(OptFile));

    if(optionsFileUsePath) {
      const char *Path = pathFind();

      if(Path != NULL) sprintf(OptFile, "%s/%s", Path, optionsFile);
    } else
      strcpy(OptFile, optionsFile);
  }

  return(OptFile);
}
/*---------------------------------------------------------------------------*/
int
optionsFileNameSet(const char *FileNm)
{
  if(access(FileNm, R_OK|W_OK) < 0) {
    printf("options file %s does not exist!\n", FileNm);
    return(-1);
  }
  optionsFileUsePath = 0;

  optionsFile = FileNm;

  return(0);
}
/*---------------------------------------------------------------------------*/
int
optionsFirstTime(void)
{
  return(options1stTime);
}
/*---------------------------------------------------------------------------*/
void
optionsFormDefault(XPMApp_t *App)
{
  FD_fmOptions *fmOp = App->Options;

  fl_set_input(fmOp->inOptionsAlbum, Options.Album);
  fl_set_input(fmOp->inOptionsDir,   Options.Dir);
  fl_set_input(fmOp->inBaseCmd,      Options.BaseCmd);
  fl_set_input(fmOp->inResetCmd,     Options.ResetCmd);
  fl_set_button(fmOp->btRandom, Options.Random);
  fl_set_button(fmOp->btRepeat, Options.Repeat);
}
/*---------------------------------------------------------------------------*/
int
optionsRandomGet(void)
{
  return(Options.Random);
}
/*---------------------------------------------------------------------------*/
void
optionsRandomSet(int Random)
{
  Options.Random = Random;
}
/*---------------------------------------------------------------------------*/
int
optionsRepeatGet(void)
{
  return(Options.Repeat);
}
/*---------------------------------------------------------------------------*/
void
optionsRepeatSet(int Repeat)
{
  Options.Repeat = Repeat;
}
/*---------------------------------------------------------------------------*/
int
optionsRead(XPMApp_t *App)
{
  register int rc = 0;
  const char *optFileNm = optionsFileNameGet();
  FILE *Fp;

  options1stTime = 0;

  if((Fp = fopen(optFileNm, "r")) == NULL) {
    printf("Could not open options file - %s\n", optFileNm);
    options1stTime = 1;
    return(optionsFileCreate());
  }

  if(fread(&Options, sizeof(Options), 1, Fp) != 1) {
    printf("Could not read options file - %s\n", optFileNm);
    options1stTime = 1;
    rc = -1;
  }
  fclose(Fp);

  return(rc);
}
/*---------------------------------------------------------------------------*/
const char *
optionsResetCmdGet(void)
{
  return(Options.ResetCmd);
}
/*---------------------------------------------------------------------------*/
void
optionsResetCmdSet(const char *ResetCmd)
{
  OPTCPY(Options.ResetCmd, ResetCmd);
}
/*---------------------------------------------------------------------------*/
void
optionsSelect(XPMApp_t *App)
{
  optionsFormDefault(App);
  fl_show_form(App->Options->fmOptions,FL_PLACE_CENTER,FL_FULLBORDER,
                 "fXPlayMidi Application Options");
}
/*---------------------------------------------------------------------------*/
int
optionsWrite(XPMApp_t *App)
{
  if(albumIsModified()) albumWrite(App);
  return(optionsFileCreate());
}
/*---------------------------------------------------------------------------*/
